
/*
    Copyright (C) 1999,2003,2007 by Wolfgang Zekoll <wzk@quietsche-entchen.de>
 
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <errno.h>
#include <wait.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <fcntl.h>

#include "lib.h"
#include "smtp.h"
#include "acp.h"




static char *readline(int fd, char *line, int size)
{
	int	len;

	*line = 0;
	
	if ((len = read(fd, line, size - 2)) < 0)
		len = 0;

	line[len] = 0;
	noctrl(line);

	close(fd);
	return (line);
}

static int getstatus(smtp_t *x, int rc, char *line, int size)
{
	int	status;

	status = atoi(line);
	if (status == 0)
		*line = 0;
	else if (status == 250  ||  status == 354) {
		if (rc != 0)
			*line = 0;
		}
	else if (status >= 400  &&  status < 600)
		/* ok */ ;
	else {
		/*
		 * Everything else is an invalid response code.
		 */

		copy_string(line, "421 server error", size);
		return (0);
		}

	if (*line == 0) {
		if (rc == 0)
			copy_string(line, "250 ok", size);
		else if (rc == 1)
			copy_string(line, "451 processing aborted", size);
		else
			copy_string(line, "554 mail rejected", size); 
		}

	return (0);
}

int run_acp(smtp_t *x, char *line, int size)
{
	int	rc, pid, pfd[2], lfd[2];

	/*
	 * Do nothing if the acp is not set.
	 */

	if (*x->config->acp == 0) {
		copy_string(line, "250 ok", size);
		return (250);
		}


	rc = 0;
	if (pipe(pfd) != 0  ||  pipe(lfd))
		printerror(1, "-ERR", "can't pipe: %s", strerror(errno));
	else if ((pid = fork()) < 0)
		printerror(1, "-ERR", "can't fork acp: %s", strerror(errno));
	else if (pid == 0) {
		int	argc;
		char	*argv[32];

		dup2(pfd[1], 2);	/* stderr nach SMTP Client */
		close(pfd[0]);

		dup2(lfd[1], 1);	/* stdout nach syslog */
		close(lfd[0]);

		close(0);

		copy_string(line, x->config->acp, size);
		argc = split(line, argv, ' ', 30);
		argv[argc] = NULL;
		execvp(argv[0], argv);

		printerror(1, "-ERR", "can't exec acp %s: %s", argv[0], strerror(errno));
		}
	else {
		/*
		 * Nicht gebrauchte fd's schliessen.
		 */

		close(pfd[1]);
		close(lfd[1]);

		readline(lfd[0], line, size);
		if (*line != 0)
			printerror(0, "", "%s", line);


		readline(pfd[0], line, size);
		if (waitpid(pid, &rc, 0) < 0)
			printerror(1, "-ERR", "error while waiting for ccp: %s", strerror(errno));

		rc = WIFEXITED(rc) != 0? WEXITSTATUS(rc): 1;
		getstatus(x, rc, line, size);
		}

	return (0);
}

int run_ccp(smtp_t *x, char *line, int size)
{
	int	rc, pid, pfd[2], lfd[2];

	/*
	 * Wenn kein ccp angegeben ist ist alles erlaubt.
	 */

	if (*x->config->ccp == 0) {
		copy_string(line, "250 ok", size);
		return (250);
		}


	rc = 0;
	if (pipe(pfd) != 0  ||  pipe(lfd))
		printerror(1, "-ERR", "can't pipe: %s", strerror(errno));
	else if ((pid = fork()) < 0)
		printerror(1, "-ERR", "can't fork ccp: %s", strerror(errno));

	else if (pid == 0) {
		int	argc;
		char	*argv[32];

		dup2(pfd[1], 2);	/* stderr nach SMTP Client */
		close(pfd[0]);

		dup2(lfd[1], 1);	/* stdout nach syslog */
		close(lfd[0]);

		close(0);

		copy_string(line, x->config->ccp, size);
		argc = split(line, argv, ' ', 30);
		argv[argc] = NULL;
		execvp(argv[0], argv);

		printerror(1, "-ERR", "can't exec ccp %s: %s", argv[0], strerror(errno));
		}
	else {
		/*
		 * Nicht gebrauchte fd's schliessen.
		 */

		close(pfd[1]);
		close(lfd[1]);


		/*
		 * syslog Meldung lesen und entsprechende pipe schliessen.
		 */

		readline(lfd[0], line, size);
		if (*line != 0)
			printerror(0, "", "%s", line);



		/*
		 * Fehlermeldung lesen, pipe schliessen.
		 */

		readline(pfd[0], line, size);


		/*
		 * return code holen.
		 */

		if (waitpid(pid, &rc, 0) < 0)
			printerror(1, "-ERR", "error while waiting for ccp: %s", strerror(errno));

		rc = WIFEXITED(rc) != 0? WEXITSTATUS(rc): 1;
		getstatus(x, rc, line, size);
		}

	return (0);
}

