
/*

    File: smtp.h
    
    Copyright (C) 1999,2000,2004,2007 by Wolfgang Zekoll <wzk@quietsche-entchen.de>

    This source is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This source is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _SMTP_INCLUDED
#define	_SMTP_INCLUDED

#include "ip-lib.h"
#include "procinfo.h"


#define	PROXYNAME	"smtp.proxy"

#define	SENDMAIL	"/usr/sbin/sendmail"
#define	SMTP_TIMEOUT	300
#define	SMTP_ACCEPTTIME	600

#define	MAX_SMTPSERVER	8
#define	SMTP_SERVERLEN	80

#define	UPTIME_FILENAME	"/proc/loadavg"
#define	EXIT_ZEROSTATUS	15

#define	SPAM_SCANCMD	"/usr/bin/spamc -c"
#define	SPAM_TAG	"[SPAM]"
#define	SPAM_MAXSIZE	(250 * 1024)


extern char *program;
extern char progname[80];
extern int debug;
extern int alsosyslog;


#define	REDIR_NONE		0
#define	REDIR_ACCEPT		1
#define	REDIR_FORWARD		2
#define	REDIR_FORWARD_ONLY	3

#define	ERRMODE_NONE		0
#define	ERRMODE_KEEP		1
#define	ERRMODE_HANDLER		2

#define	ERR_EXITCODEMASK	15

#define	ERR_STDERR		0x00000080

#define	ERR_ACCESS		0x00000100
#define	ERR_CONNECT		0x00000200
#define	ERR_TIMEOUT		0x00000400
#define	ERR_SERVER		0x00000800
#define	ERR_CLIENT		0x00001000
#define	ERR_PROXY		0x00002000
#define	ERR_INFO		0x00004000
#define	ERR_OK			0x00080000
#define	ERR_ANY			0x0FFFFF00

#define	RBL_REJECT		0
#define	RBL_LOGONLY		1
#define	RBL_QUARANTINE		2
#define	RBL_DELAYED		3

#define	DIR_AUTOSUBDIR		0
#define	DIR_PLAIN		1

#define	INDEX_INFO		0
#define	INDEX_TABLE		1
#define	INDEX_NONE		2

#define	SM_PRIMSEC		0
#define	SM_ROUNDROBIN		1


#define	WAITING		1
#define	MAIL_SEEN	2
#define	RCPT_SEEN	3
#define	DATA_SEEN	4
#define	SEND_QUIT	5
#define	NO_SENDMAIL	6

#define	UNKNOWNCOMMAND_LIMIT	5

typedef struct _tokentab {
    char	word[20];
    int		opc;
    } tokentab_t;

extern tokentab_t rblmodetab[], dirmodetab[], indexmodetab[];

typedef struct _qdir {
    char	dir[200];
    int		dirmode;
    int		indexstyle;
    } qdir_t;

typedef struct _config {
    char	configfile[200];
    char	interface[80];
    int		haveinterfaces;
    int		subconfig;
 
    char	hostname[200];
    int		etrn;
    int		droppath;
    int		timeout;
    int		scanmail;
    char	loadavg[80];

    struct {
	char	controlfile[200];
	char	filename[200];
	} debug;

    struct {
	int	permitted;
	} allowdeny;

    struct {
	int	passonerror;
	int	passonvirus;
	qdir_t	qd;
	int	statuscode;
	} clamav;

    struct {
	int	scan;
	int	quarantine;
	qdir_t	qd;
	int	treshhold;
	int	statuscode;
	char	spamtag[80];
	char	cmd[200];
	} spamd;

    struct {
	int		usefirst;
	int		verbose;
	int		mode;
	string_t	list;
	} rbl;

    struct {
	int	timeout;
	int	rc;
	int	delay;
	} earlytalker;

/*    char	senderlist[1024]; */
/*    char	rcptlist[1024]; */
    string_t	senderlist;
    string_t	rcpt;
    char	senderdomain[200];

    char	srcip[80];
/*    char	server[300]; */
    struct {
	int	count;
	char	server[MAX_SMTPSERVER][SMTP_SERVERLEN];

	int	mode;
	char	laststartfile[200];
	} server;

    char	**argv;

    char	varname[80];
    char	acp[200];
    char	ccp[200];

    char	clientdir[200];
    int		accepttime;

    char	pidfile[200];

    struct {
	char    name[40];
	unsigned int uid, gid;
	} user;
    } config_t;



typedef struct _bio {
    char	name[40];
    int		fd, out;

    int		here, len;
    char	buffer[1080];
    } bio_t;


typedef struct _smtp {
    config_t	*config;

    peer_t	interface;
    peer_t	s, origdst;

    char	servername[200];

    char	hostname[200];

    bio_t	sbio;
    bio_t	cbio;

    char	ipnum[80];
    char	client[200];

    int		helloseen;
    int		state;
    int		mailcount;
    char	lastcmd[80];


	/*
	 * Mail-spezifische Daten
	 */

    char	origsender[300];
    char	sender[300];
    int		nrcpt;
    struct {
	int	here, max;
	char	*list;
	} rcpt;

    char	subject[200];
    char	msgid[300];
    long	size;

    struct {
	char	adress[80];
	unsigned int port;
	char	version[200];
	bio_t	bio;
/*	string_t header; */
	} clamav;

    struct {
	int	blacklisted;
	char	resp[300];
	} rbl;

    unsigned long lasttimer;
    unsigned int keepalivecounter;

    char	spoolfile[200];
    unsigned long started;
    } smtp_t;


extern int printerror(int rc, char *type, char *format, ...);
extern int checkload(config_t *config, char *loadpar, char *current, int size);
extern int writestatfile(smtp_t *x, char *status);

extern int spooldata(smtp_t *x, FILE *fp, char *tag);
extern int readconfig(config_t *config, char *filename, char *section, char *ipnum);
extern int proxy_request(config_t *config);

extern int checkrbl(smtp_t *x, char *ipnum, char *resp, int size);

extern char *gettmpdir();
extern int settmpdir(char *dir);

extern int createdir(char *dir);
extern int doscanmail(smtp_t *x, char *resp, int size);

extern int getc_fd(smtp_t *x, bio_t *bio, int nooptime);
extern char *readline_fd(smtp_t *x, bio_t *bio, char *line, int size, int nooptime, int cutblanks);
extern char *cfgets(smtp_t *x, char *line, int size, int nooptime, int debuglevel);
extern int cfputs(smtp_t *x, char *line);
extern char *sfgets(smtp_t *x, char *line, int size);
extern int sfputs(smtp_t *x, int debuglevel, char *format, ...);
extern int sfputc(smtp_t *x, char *command, char *parameter, char *line, int size, char **here);
extern int proxy_request(config_t *config);

#endif

